import { ApiProperty } from '@nestjs/swagger';
import { Contract } from 'src/mysqlcompany/contract/entities/contract.entity';
import { Department } from 'src/mysqlcompany/department/entities/department.entity';
import {
  Column,
  Entity,
  JoinColumn,
  JoinTable,
  ManyToOne,
  OneToOne,
  PrimaryGeneratedColumn,
} from 'typeorm';

@Entity('staffs')
export class Staff {
  @ApiProperty({
    type: Number,
    name: 'id',
    description: 'id of staff, auto-incremented',
    required: true,
  })
  @PrimaryGeneratedColumn('increment')
  id: number;

  @ApiProperty({
    type: String,
    name: 'name',
    description: 'Staff name',
    required: true,
  })
  @Column({
    type: 'varchar',
  })
  name: string;

  @ApiProperty({
    type: Number,
    name: 'staffCode',
    description: 'Staff code/ID',
    required: false,
  })
  @Column({
    name: 'staff_code',
    type: 'varchar',
    length: 4,
    nullable: true,
  })
  staffCode: number;

  @ApiProperty({
    name: 'department',
    description: 'The Department that the staff is undered',
    type: () => Department,
    required: false,
  })
  @ManyToOne(() => Department, (department) => department.staffs, {
    nullable: false,
    eager: true,
  })
  // @JoinColumn({ name: 'department_id', referencedColumnName: 'id' })
  @JoinColumn({ name: 'department_id' })
  department: Department;

  @ApiProperty({
    name: 'contract',
    description: 'The staff contract',
    type: () => Contract,
  })
  @OneToOne(() => Contract, (contract) => contract.staff, {
    nullable: false,
    eager: true,
    cascade: true,
  })
  @JoinColumn({ name: 'contract_id' })
  contract: Contract;
}
