import { Injectable, Logger } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { EntityNotFoundError, Repository } from 'typeorm';
import { Department } from './entities/department.entity';
import { DepartmentCreateDto } from './dto/department-create.dto';
import { DepartmentUpdateDto } from './dto/department-update.dto';

@Injectable()
export class DepartmentService {
  constructor(
    private readonly logger: Logger,
    @InjectRepository(Department)
    private departmentRepository: Repository<Department>,
  ) {}

  create(departmentCreateDto: DepartmentCreateDto): Promise<Department> {
    // Use Repository.create() will copy the values to destination object.
    const department = this.departmentRepository.create(departmentCreateDto);
    return this.departmentRepository.save(department);
  }

  findAll(): Promise<Department[]> {
    return this.departmentRepository.find();
  }

  findOne(id: number): Promise<Department> {
    return this.departmentRepository.findOneOrFail({ where: { id } });
  }

  async update(
    id: number,
    departmentUpdateDto: DepartmentUpdateDto,
  ): Promise<void> {
    await this.isExist(id);
    await this.departmentRepository.update({ id }, departmentUpdateDto);
  }

  async remove(id: number): Promise<void> {
    await this.isExist(id);
    await this.departmentRepository.delete(id);
  }

  // Helper function: Check if the entity exist.
  // If entity does not exsit, return the Promise.reject()
  private async isExist(id: number): Promise<void> {
    const cnt = await this.departmentRepository.countBy({ id });
    if (cnt > 0) {
      return;
    } else {
      return Promise.reject(
        new EntityNotFoundError(Department, { where: { id } }),
      );
    }
  }
}
