import { ApiProperty } from '@nestjs/swagger';
import { Department } from 'src/mysqlcompany/department/entities/department.entity';
import {
  Column,
  Entity,
  JoinColumn,
  JoinTable,
  ManyToOne,
  PrimaryGeneratedColumn,
} from 'typeorm';

@Entity('staffs')
@Entity()
export class Staff {
  @ApiProperty({
    type: Number,
    name: 'id',
    description: 'id of staff, auto-incremented',
    required: true,
  })
  @PrimaryGeneratedColumn('increment')
  id: number;

  @ApiProperty({
    type: String,
    name: 'name',
    description: 'Staff name',
    required: true,
  })
  @Column({
    type: 'varchar',
  })
  name: string;

  @ApiProperty({
    type: Number,
    name: 'staffCode',
    description: 'Staff code/ID',
    required: false,
  })
  @Column({
    name: 'staff_code',
    type: 'varchar',
    length: 10,
    nullable: true,
  })
  staffCode: number;

  @ApiProperty({
    type: Number,
    name: 'departmentId',
    description: 'Department ID',
    required: false,
  })
  @Column({
    // Match existing column naming convention
    name: 'department_id',
    type: 'tinyint',
    unsigned: true,
    nullable: true,
  })
  departmentId: number;

  @ManyToOne(() => Department)
  @JoinColumn({ name: 'department_id' })
  department: Department;
}
